# -*- coding: utf-8 -*-
#
# Copyright (©) 2013 Marcelo Jorge Vieira <metal@alucinados.com>
# Copyright (©) 2013 Gustavo Noronha Silva <gustavo@noronha.eti.br>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import locale
import json
from colorsys import hsv_to_rgb
from datetime import date
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.shortcuts import render as original_render
from django.db.models import Sum, Q
from django.http import HttpResponse, Http404
from django.core.mail import send_mail
from django.conf import settings
from montanha.models import *
from montanha.forms import *
from montanha.util import (
    filter_for_institution, get_date_ranges_from_data, ensure_years_in_range
)

locale.setlocale(locale.LC_MONETARY, "pt_BR.UTF-8")
locale.setlocale(locale.LC_TIME, "pt_BR.UTF-8")


def generate_colors(n=7, sat=1.0, val=1.0):
    '''Generates an array of n colors from red to violet, considering
    saturation sat and value val'''
    return ['#%02X%02X%02X' % t for t in [tuple([int(round(c * 255)) for c in t]) for t in
           [hsv_to_rgb(x/float(n), sat, val) for x in xrange(0, n)]]]


def new_render(request, filter_spec, template, context):
    context['institution'] = None
    context['legislature'] = None
    context['filter_spec'] = None
    if filter_spec:
        institution, legislature = parse_filter(filter_spec)
        context['institution'] = institution
        context['legislature'] = legislature
        context['filter_spec'] = filter_spec
    return original_render(request, template, context)


def get_basic_objects_for_model(filter_spec, model=Expense):
    data = model.objects.all()

    if not filter_spec:
        date_ranges = get_date_ranges_from_data(None, data)
        return data, date_ranges

    institution, legislature = parse_filter(filter_spec)
    if institution:
        data = filter_for_institution(data, institution)

    if legislature:
        data = data.filter(mandate__legislature=legislature)

    date_ranges = get_date_ranges_from_data(institution, data)

    return data, date_ranges


def parse_filter(filter_spec):
    if not filter_spec:
        return None, None

    parts = filter_spec.split(':', 1)

    try:
        institution = Institution.objects.get(siglum=parts[0])
    except Institution.DoesNotExist:
        return None, None

    if len(parts) == 1:
        return institution, None

    try:
        legislature = institution.legislature_set.get(date_start__year=parts[1])
    except ValueError:
        raise Http404

    return institution, legislature


def show_index(request, filter_spec):

    c = {}

    if filter_spec:
        institution, _ = parse_filter(filter_spec)
        if not institution:
            raise Http404
        legislatures = institution.legislature_set.order_by('-date_start').all()
        c['legislatures'] = [l for l in legislatures
                             if Expense.objects.filter(mandate__legislature=l).count()]
        c['img'] = institution.siglum.lower() + '.png'
        return new_render(request, filter_spec, 'institution.html', c)

    data = BiggestSupplierForYear.objects.filter(year=date.today().year)

    # Empty-named supplier appears to be used for expenses done through the house?
    data = data.exclude(supplier__name='').order_by('-expensed')
    if data.count() > 8:
        data = data[:8]

    c = {'biggest_suppliers': data}

    return new_render(request, filter_spec, 'index.html', c)


def show_robots_txt(request):
    return HttpResponse('User-Agent: *\nAllow: /\n')


def error_500(request):
    c = {}
    return original_render(request, '500.html', c)


def error_404(request):
    c = {}
    return original_render(request, '404.html', c)


def show_per_nature(request, filter_spec):

    institution, legislature = parse_filter(filter_spec)
    data = PerNature.objects.filter(institution=institution, legislature=legislature)

    date_ranges = get_date_ranges_from_data(institution, data, consolidated_data=True)

    time_series = []
    for d in data:
        l = []
        cummulative = .0
        time_series.append(dict(label=d.nature.name, data=l))

        per_year_data = PerNatureByYear.objects.filter(institution=institution).filter(nature=d.nature)
        per_year_data = per_year_data.filter(year__gte=date_ranges['cdf'].year)
        per_year_data = per_year_data.filter(year__lte=date_ranges['cdt'].year)
        for item in per_year_data:
            cummulative = cummulative + float(item.expensed)
            l.append([int(date(item.year, 1, 1).strftime("%s000")), cummulative])

    # mbm_years = list of years (3, right now - 2011, 2012, 2013) with 12 months inside
    # each month in turn carries a dict with month name and value or null.
    def nature_is_empty(years):
        for year in years:
            for month in year['data']:
                if month['data'] != 'null' and month['data'] > 0.0001:
                    return False
        return True

    natures_mbm = []
    for nature in ExpenseNature.objects.all():
        mbm_years = []
        expensed_by_month = PerNatureByMonth.objects.filter(institution=institution)
        expensed_by_month = expensed_by_month.filter(nature=nature)

        all_years = [d.year for d in expensed_by_month.dates('date', 'year')]
        all_years = ensure_years_in_range(date_ranges, all_years)

        for year in all_years:
            mbm_series = []
            for month in range(1, 13):
                month_date = date(year, month, 1)

                mname = month_date.strftime("%b")
                try:
                    item = expensed_by_month.get(date=month_date)
                    expensed = float(item.expensed)
                except PerNatureByMonth.DoesNotExist:
                    expensed = 'null'

                mbm_series.append(dict(month=mname, data=str(expensed)))
            mbm_years.append(dict(label=year, data=mbm_series))

        if not nature_is_empty(mbm_years):
            natures_mbm.append(dict(name=nature.name, data=mbm_years))

    c = {'data': data, 'years_data': time_series, 'natures_mbm': natures_mbm,
         'colors': generate_colors(len(data), 0.93, 0.8)}

    c.update(date_ranges)

    return new_render(request, filter_spec, 'per_nature.html', c)


def show_per_legislator(request, filter_spec):

    institution, legislature = parse_filter(filter_spec)
    data = PerLegislator.objects.filter(institution=institution, legislature=legislature)

    date_ranges = get_date_ranges_from_data(institution, data, consolidated_data=True)

    # We may have several mandates for a legislator, potentially with different parties.
    # We only want to show one line per legislator, though, so sum the expenses and list
    # all parties in the party column.
    data_list = []
    for item in data:
        mandates = item.legislator.mandate_set.order_by('date_start')
        legislator_data = dict(legislator=item.legislator,
                               mandates=mandates,
                               expensed=item.expensed)
        data_list.append(legislator_data)
    data = data_list

    c = {'data': data}

    c.update(date_ranges)

    return new_render(request, filter_spec, 'per_legislator.html', c)


def show_legislator_detail(request, filter_spec, legislator_id):

    data, date_ranges = get_basic_objects_for_model(filter_spec)

    legislator = Legislator.objects.get(pk=legislator_id)
    data = data.filter(mandate__legislator=legislator)

    top_suppliers = data.values('supplier__id',
                                'supplier__identifier',
                                'supplier__name')
    top_suppliers = top_suppliers.annotate(expensed=Sum('expensed')).order_by('-expensed')
    top_suppliers = top_suppliers[:15]

    total_expensed = data.values('supplier__name')
    total_expensed = total_expensed.annotate(total_expensed=Sum('expensed'))
    if len(total_expensed) > 0:
        total_expensed = total_expensed[0]['total_expensed']
    else:
        total_expensed = 0

    data = data.values('nature__name', 'supplier__name', 'supplier__identifier',
                       'number', 'date', 'expensed').order_by('-date')

    paginator = Paginator(data, 10)
    page = request.GET.get('page')
    try:
        data = paginator.page(page)
    except PageNotAnInteger:
        data = paginator.page(1)
    except EmptyPage:
        data = paginator.page(paginator.num_pages)

    c = {'legislator': legislator, 'data': data, 'top_suppliers': top_suppliers}

    c.update(date_ranges)

    return new_render(request, filter_spec, 'detail_legislator.html', c)


def postprocess_party_data(institution, data):
    if institution:
        if not isinstance(institution, Institution):
            institution = Institution.objects.get(siglum=institution)

    for d in list(data):
        if d['mandate__party__siglum']:
            party = PoliticalParty.objects.get(siglum=d['mandate__party__siglum'])
            mandates = party.mandate_set.all()
            if institution:
                mandates = mandates.filter(legislature__institution=institution)
            d['n_legislators'] = mandates.values('legislator').count()
            d['expensed_average'] = d['expensed'] / d['n_legislators']
        else:
            d['mandate__party__siglum'] = 'NC'
            d['mandate__party__name'] = 'Desconhecido'
            mandates = Mandate.objects.filter(party__siglum=None)
            if institution:
                mandates = mandates.filter(legislature__institution=institution)
            d['n_legislators'] = mandates.values('legislator').count()
            if d['n_legislators']:
                d['expensed_average'] = d['expensed'] / d['n_legislators']
            else:
                d['expensed_average'] = 0.

    return sorted(data, key=lambda d: d['expensed_average'], reverse=True)


def show_per_party(request, filter_spec):

    institution, _ = parse_filter(filter_spec)
    data, date_ranges = get_basic_objects_for_model(filter_spec)

    data = data.values('mandate__party__logo', 'mandate__party__siglum', 'mandate__party__name')
    data = data.annotate(expensed=Sum('expensed'))

    data = postprocess_party_data(institution, data)

    c = {'data': data, 'graph_data': data, 'colors': generate_colors(len(data), 0.93, 0.8)}

    c.update(date_ranges)

    return new_render(request, filter_spec, 'per_party.html', c)


def add_sorting(request, data, default='-expensed'):
    if 'order_by' in request.GET:
        order_by_field = request.GET.get('order_by')
        if not 'asc' in request.GET or not request.GET.get('asc'):
            order_by_field = '-' + order_by_field
        data = data.order_by(order_by_field)
    else:
        data = data.order_by(default)
    return data


def show_per_supplier(request, filter_spec):

    data, date_ranges = get_basic_objects_for_model(filter_spec)

    data = data.values('supplier__id', 'supplier__name', 'supplier__identifier')
    data = data.annotate(expensed=Sum('expensed'))

    data = add_sorting(request, data)

    paginator = Paginator(data, 10)
    page = request.GET.get('page')
    try:
        data = paginator.page(page)
    except PageNotAnInteger:
        data = paginator.page(1)
    except EmptyPage:
        data = paginator.page(paginator.num_pages)

    c = {'data': data}

    c.update(date_ranges)

    return new_render(request, filter_spec, 'per_supplier.html', c)


def get_supplier_detail_data(request, supplier_id, filter_spec=''):
    institution, _ = parse_filter(filter_spec)
    data, date_ranges = get_basic_objects_for_model(filter_spec)

    supplier = Supplier.objects.get(pk=supplier_id)
    data = data.filter(supplier=supplier)

    # Data prepared for displaying the per-party graph
    graph_data = data.values('mandate__party__logo', 'mandate__party__siglum', 'mandate__party__name')
    graph_data = graph_data.annotate(expensed=Sum('expensed'))
    graph_data = postprocess_party_data(institution, graph_data)

    top_buyers = data.values('mandate__legislator__id',
                             'mandate__legislator__name',
                             'mandate__legislature__institution__siglum',
                             'mandate__party__siglum')
    top_buyers = top_buyers.annotate(expensed=Sum('expensed')).order_by('-expensed')
    top_buyers = top_buyers[:15]

    total_expensed = data.values('supplier__name')
    total_expensed = total_expensed.annotate(total_expensed=Sum('expensed'))
    if len(total_expensed) > 0:
        total_expensed = total_expensed[0]['total_expensed']
    else:
        total_expensed = 0

    data = data.values('nature__name',
                       'mandate__legislator__name', 'mandate__party__siglum',
                       'number', 'date', 'expensed').order_by('-date')

    paginator = Paginator(data, 10)
    page = request.GET.get('page')
    try:
        data = paginator.page(page)
    except PageNotAnInteger:
        data = paginator.page(1)
    except EmptyPage:
        data = paginator.page(paginator.num_pages)

    c = {'supplier': supplier,
         'data': data,
         'graph_data': graph_data,
         'top_buyers': top_buyers,
         'total_expensed': total_expensed,
         'colors': generate_colors(len(data), 0.93, 0.8)}

    c.update(date_ranges)
    return c


def show_supplier_overview(request, supplier_id):
    c = get_supplier_detail_data(request, supplier_id)

    data = Expense.objects.filter(supplier__id=supplier_id)

    total_expensed = data.aggregate(e=Sum('expensed'))['e']

    data = data.values('mandate__legislature__institution__logo',
                       'mandate__legislature__institution__siglum',
                       'mandate__legislature__institution__name')
    data = data.annotate(expensed=Sum('expensed'))
    data = data.order_by('-expensed')

    house_data = list()
    for d in data:
        if d['expensed'] > 0:
            proportion = total_expensed / d['expensed']
        else:
            proportion = 0

        house_data.append(dict(
            logo=d['mandate__legislature__institution__logo'],
            siglum=d['mandate__legislature__institution__siglum'],
            name=d['mandate__legislature__institution__name'],
            expensed=d['expensed'],
            proportion=proportion
        ))

    c['total_expensed'] = total_expensed
    c['house_data'] = house_data

    return new_render(request, '', 'supplier_overview.html', c)


def show_supplier_detail(request, filter_spec, supplier_id):
    c = get_supplier_detail_data(request, supplier_id, filter_spec)
    return new_render(request, filter_spec, 'detail_supplier.html', c)


def deep_getattr(obj, attr):
    return reduce(getattr, attr.split('.'), obj)


def convert_data_to_list(data, columns):
    data_list = []
    for item in data:
        sublist = []
        for field, field_type in columns:
            value = deep_getattr(item, field)

            if field_type == 'm':
                value = locale.currency(value, grouping=True).replace(" ", "&nbsp;")

            if isinstance(value, date):
                value = value.strftime('%d/%m/%Y')

            sublist.append(value)
        data_list.append(sublist)
    return data_list


def data_tables_query(request, filter_spec, columns, filter_function=None):

    data, date_ranges = get_basic_objects_for_model(filter_spec)

    if filter_function:
        data = filter_function(data)

    # json doesn't like to serialize date objects
    del date_ranges['cdf']
    del date_ranges['cdt']

    total_results = data.count()

    search_string = request.GET.get('sSearch', None)
    if search_string:
        try:
            search_string = search_string.decode('utf-8')
        except UnicodeEncodeError:
            pass

        def format_for_column_type(col_type, search_string):
            if col_type == 'm':
                search_string = search_string.replace('.', '').replace(',', '.')
                search_string = search_string.replace('R$', '')
                return search_string.strip()
            return search_string

        filter_expression = None
        for name, col_type in columns:
            if col_type == 'd':
                continue

            actual_search_string = format_for_column_type(col_type, search_string)
            exp = Q(**{name.replace('.', '__') + '__icontains': actual_search_string})
            if filter_expression:
                filter_expression |= exp
            else:
                filter_expression = exp
        data = data.filter(filter_expression)

    displayed_results = data.count()

    # Sort by expense by default, assumed to be the last column.
    sort_col = int(request.GET.get('iSortCol_0', len(columns) - 1))
    order_by_field = columns[sort_col][0].replace('.', '__')

    sort_direction = request.GET.get('sSortDir_0', 'asc')
    if sort_direction == 'desc':
        order_by_field = '-' + order_by_field

    data = data.order_by(order_by_field)

    per_page = int(request.GET.get('iDisplayLength', 10))
    starting_at = int(request.GET.get('iDisplayStart', 0))
    page = starting_at > 0 and starting_at / per_page + 1 or 1

    paginator = Paginator(data, per_page)
    try:
        data = paginator.page(page)
    except PageNotAnInteger:
        data = paginator.page(1)
    except EmptyPage:
        data = paginator.page(paginator.num_pages)

    data = convert_data_to_list(data, columns)

    response = dict(sEcho=int(request.GET.get('sEcho', 0)),
                    iTotalRecords=total_results,
                    iTotalDisplayRecords=displayed_results,
                    aaData=data)

    response.update(date_ranges)

    return HttpResponse(json.dumps(response), content_type='application/json')


def query_all(request, filter_spec):
    columns = (
        ('nature.name', 's'),
        ('mandate.legislator.name', 's'),
        ('supplier.name', 's'),
        ('supplier.identifier_with_mask', 's'),
        ('number', 's'),
        ('date', 'd'),
        ('expensed', 'm'),
    )

    return data_tables_query(request, filter_spec, columns)


def query_supplier_all(request, filter_spec):
    def filter_function(data):
        supplier_id = request.GET.get('item_id', None)
        if supplier_id:
            try:
                supplier = Supplier.objects.get(pk=supplier_id)
                return data.filter(supplier=supplier)
            except Supplier.DoesNotExist:
                return data
        return data

    columns = (
        ('nature.name', 's'),
        ('mandate.legislator.name', 's'),
        ('number', 's'),
        ('date', 'd'),
        ('expensed', 'm'),
    )

    return data_tables_query(request, filter_spec, columns, filter_function)


def query_legislator_all(request, filter_spec):
    def filter_function(data):
        legislator_id = request.GET.get('item_id', None)
        if legislator_id:
            try:
                legislator = Legislator.objects.get(pk=legislator_id)
                return data.filter(mandate__legislator=legislator)
            except Legislator.DoesNotExist:
                return data
        return data

    columns = (
        ('nature.name', 's'),
        ('supplier.name', 's'),
        ('supplier.identifier_with_mask', 's'),
        ('number', 's'),
        ('date', 'd'),
        ('expensed', 'm'),
    )

    return data_tables_query(request, filter_spec, columns, filter_function)


def show_all(request, filter_spec):

    c = {}

    _, date_ranges = get_basic_objects_for_model(filter_spec)

    c.update(date_ranges)

    return new_render(request, filter_spec, 'all_expenses.html', c)


def what_is_expenses(request):

    c = {}

    return original_render(request, 'what_is_expenses.html', c)


def contact_us(request):

    contact_us_form = ContactUsForm(request.POST or None)
    success_message = ''

    if request.POST and contact_us_form.is_valid():
        subject = '[Olho Neles]: Fale Conosco'

        message = ('Nome: %s\nEmail: %s\nIP: %s\nMensagem:\n\n%s') % (
            contact_us_form.cleaned_data['name'],
            contact_us_form.cleaned_data['email'],
            request.META['REMOTE_ADDR'],
            contact_us_form.cleaned_data['message'])

        from_field = '%s <%s>' % (contact_us_form.cleaned_data['name'],
                                  contact_us_form.cleaned_data['email'])

        send_mail(subject, message, from_field, [settings.CONTACT_US_EMAIL])

        success_message = ("""Sua mensagem foi enviada com sucesso. """
                           """Em breve entraremos em contato!""")

        contact_us_form = ContactUsForm(None)

    c = {'contact_us_form': contact_us_form,
         'success_message': success_message}

    return original_render(request, 'contact_us.html', c)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'FAQ'
        db.create_table(u'cms_faq', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(default=1, to=orm['sites.Site'])),
            ('question', self.gf('django.db.models.fields.TextField')(max_length=1000)),
            ('answer', self.gf('django.db.models.fields.TextField')(max_length=5000)),
            ('last_change', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'cms', ['FAQ'])


    def backwards(self, orm):
        # Deleting model 'FAQ'
        db.delete_table(u'cms_faq')


    models = {
        u'cms.faq': {
            'Meta': {'object_name': 'FAQ'},
            'answer': ('django.db.models.fields.TextField', [], {'max_length': '5000'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_change': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'question': ('django.db.models.fields.TextField', [], {'max_length': '1000'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['sites.Site']"})
        },
        u'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['cms']


# Django settings for olhoneles project.

import os
import sys
from derpconf.config import Config

INSTANCE = lambda *x: os.path.join(os.path.dirname(__file__), *x)

if 'test' in sys.argv:
    config_file = INSTANCE('tests.config')
else:
    config_file = INSTANCE('local.config')

if os.path.isfile(config_file):
    conf = Config.load(config_file)
else:
    conf = Config()

DEBUG = conf.get('DEBUG', True)
TEMPLATE_DEBUG = DEBUG

ADMINS = conf.get('ADMINS', ())

MANAGERS = ADMINS

DEAFAULT_DATABASE = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': INSTANCE('db.sqlite3'),
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

DATABASES = conf.get('DATABASES', DEAFAULT_DATABASE)

# Hosts/domain names that are valid for this site; required if DEBUG is False
# See https://docs.djangoproject.com/en/1.5/ref/settings/#allowed-hosts
ALLOWED_HOSTS = conf.get('ALLOWED_HOSTS', [])

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = conf.get('TIME_ZONE', 'America/Chicago')

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = conf.get('LANGUAGE_CODE', 'en-us')

SITE_ID = conf.get('SITE_ID', 1)

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = conf.get('USE_I18N', True)

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = conf.get('USE_L10N', True)

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = conf.get('USE_TZ', True)

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/var/www/example.com/media/"
MEDIA_ROOT = conf.get('MEDIA_ROOT', INSTANCE('../media'))

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://example.com/media/", "http://media.example.com/"
MEDIA_URL = conf.get('MEDIA_URL', '/media/')

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/var/www/example.com/static/"
STATIC_ROOT = conf.get('STATIC_ROOT', '')

# URL prefix for static files.
# Example: "http://example.com/static/", "http://static.example.com/"
STATIC_URL = conf.get('STATIC_URL', '/static/')

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = conf.get('SECRET_KEY', 'REPLACE-THIS-IN-CONFIG-LOCAL')

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    # 'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'olhoneles.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'olhoneles.wsgi.application'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

DEFAULT_INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'bootstrap_toolkit',
    'easy_thumbnails',
    'montanha',
    'cms',
    'captcha',
    'raven.contrib.django.raven_compat',
    # South must be the last one, since it is used for migrating models of
    # all apps, potentially.
    'south',
)

INSTALLED_APPS = conf.get('INSTALLED_APPS', DEFAULT_INSTALLED_APPS)

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'root': {
        'level': 'WARNING',
        'handlers': ['sentry'],
    },
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
    },
    'handlers': {
        'sentry': {
            'level': 'ERROR',
            'class': 'raven.contrib.django.raven_compat.handlers.SentryHandler',
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        }
    },
    'loggers': {
        'django.db.backends': {
            'level': 'ERROR',
            'handlers': ['console'],
            'propagate': False,
        },
        'raven': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
        'sentry.errors': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
    },
}

THUMBNAIL_ALIASES = {
    '': {
        'mini': {'size': (20, 20), 'crop': False},
    },
}

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.tz',
    'django.core.context_processors.request',
)

DEFAULT_CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
        'LOCATION': INSTANCE('../cache'),
    }
}

CACHES = conf.get('CACHES', DEFAULT_CACHES)

DEFAULT_FROM_EMAIL = conf.get('DEFAULT_FROM_EMAIL', 'montanha-dev@listas.olhoneles.org')

CONTACT_US_EMAIL = conf.get('CONTACT_US_EMAIL', 'montanha@olhoneles.org')

SERVER_EMAIL = conf.get('SERVER_EMAIL', 'montanha@olhoneles.org')

RECAPTCHA_PUBLIC_KEY = conf.get('RECAPTCHA_PUBLIC_KEY', '')
RECAPTCHA_PRIVATE_KEY = conf.get('RECAPTCHA_PRIVATE_KEY', '')

RAVEN_CONFIG = conf.get('RAVEN_CONFIG', {'dsn': ''})



# Downloaded from http://wiki.python.org.br/Cnpj
# License: CC-BY http://creativecommons.org/licenses/by/2.5/br/


class Cnpj:
    def __init__( self ):
        """
        Class to interact with Cnpj brazilian numbers
        """
        pass

    def validate( self, cnpj ):
        """
        Method to validate brazilian cnpjs
        Tests:

        >>> print Cnpj().validate('61882613000194')
        True
        >>> print Cnpj().validate('61882613000195')
        False
        >>> print Cnpj().validate('53.612.734/0001-98')
        True
        >>> print Cnpj().validate('69.435.154/0001-02')
        True
        >>> print Cnpj().validate('69.435.154/0001-01')
        False
        """
        # defining some variables
        lista_validacao_um = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4 , 3, 2]
        lista_validacao_dois = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]

        # cleaning the cnpj
        cnpj = cnpj.replace( "-", "" )
        cnpj = cnpj.replace( ".", "" )
        cnpj = cnpj.replace( "/", "" )

        # finding out the digits
        verificadores = cnpj[-2:]

        # verifying the lenght of the cnpj
        if len( cnpj ) != 14:
            return False

        # calculating the first digit
        soma = 0
        id = 0
        for numero in cnpj:

            # to do not raise indexerrors
            try:
                lista_validacao_um[id]
            except:
                break

            soma += int( numero ) * int( lista_validacao_um[id] )
            id += 1

        soma = soma % 11
        if soma < 2:
            digito_um = 0
        else:
            digito_um = 11 - soma

        digito_um = str( digito_um ) # converting to string, for later comparison

        # calculating the second digit
        # suming the two lists
        soma = 0
        id = 0

        # suming the two lists
        for numero in cnpj:

            # to do not raise indexerrors
            try:
                lista_validacao_dois[id]
            except:
                break

            soma += int( numero ) * int( lista_validacao_dois[id] )
            id += 1

        # defining the digit
        soma = soma % 11
        if soma < 2:
            digito_dois = 0
        else:
            digito_dois = 11 - soma

        digito_dois = str( digito_dois )

        # returnig
        return bool( verificadores == digito_um + digito_dois )

    def format( self, cnpj ):
        """
        Method to format cnpj numbers.
        Tests:

        >>> print Cnpj().format('53612734000198')
        53.612.734/0001-98
        """
        return "%s.%s.%s/%s-%s" % ( cnpj[0:2], cnpj[2:5], cnpj[5:8], cnpj[8:12], cnpj[12:14] )



# -*- coding: utf-8 -*-
#
# Copyright (©) 2014 Gustavo Noronha Silva
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This hack makes django less memory hungry (it caches queries when running
# with debug enabled.
from django.conf import settings
settings.DEBUG = False

import codecs
import sys
from datetime import date
from django.core.management.base import BaseCommand
from django.db.models import Sum
from montanha.models import Institution, Expense, ExpenseNature, Legislator, Supplier
from montanha.models import PerNature, PerNatureByYear, PerNatureByMonth, PerLegislator, BiggestSupplierForYear
from montanha.util import filter_for_institution, get_date_ranges_from_data, ensure_years_in_range


sys.stdout = codecs.getwriter("utf-8")(sys.stdout)
sys.stderr = codecs.getwriter("utf-8")(sys.stderr)


class Command(BaseCommand):
    args = u"<source> [source2] … [sourcen]"
    help = "Collects data for a number of sources"

    def handle(self, *args, **options):
        institutions = []
        if "almg" in args:
            institutions.append(Institution.objects.get(siglum='ALMG'))

        if "algo" in args:
            institutions.append(Institution.objects.get(siglum='ALGO'))

        if "senado" in args:
            institutions.append(Institution.objects.get(siglum='Senado'))

        if "cmbh" in args:
            institutions.append(Institution.objects.get(siglum='CMBH'))

        if "cmsp" in args:
            institutions.append(Institution.objects.get(siglum='CMSP'))

        if "cdep" in args:
            institutions.append(Institution.objects.get(siglum='CDF'))

        for institution in institutions:
            print u'Consolidating data for %s' % (institution.name)

            # Per nature
            PerNature.objects.filter(institution=institution).delete()
            PerNatureByYear.objects.filter(institution=institution).delete()
            PerNatureByMonth.objects.filter(institution=institution).delete()

            data = Expense.objects.all()
            data = filter_for_institution(data, institution)

            date_ranges = get_date_ranges_from_data(institution, data)

            data = data.values('nature__id')
            data = data.annotate(expensed=Sum('expensed')).order_by('-expensed')

            years = [d.year for d in Expense.objects.dates('date', 'year')]
            years = ensure_years_in_range(date_ranges, years)

            per_natures_to_create = list()
            per_natures_by_year_to_create = list()
            per_natures_by_month_to_create = list()

            for item in data:
                # Totals
                nature = ExpenseNature.objects.get(id=item['nature__id'])
                p = PerNature(institution=institution,
                              date_start=date_ranges['cdf'],
                              date_end=date_ranges['cdt'],
                              nature=nature,
                              expensed=item['expensed'])
                per_natures_to_create.append(p)

                # Totals for Legislature
                for legislature in institution.legislature_set.all():
                    pargs = (institution.siglum, nature.name, legislature.date_start.year, legislature.date_end.year)
                    print u'[%s] Consolidating nature %s totals for legislature %d-%d…' % pargs
                    legislature_data = Expense.objects.filter(nature=nature)
                    legislature_data = legislature_data.filter(mandate__legislature=legislature)

                    legislature_ranges = get_date_ranges_from_data(institution, legislature_data)

                    legislature_data = legislature_data.values('nature__id')
                    legislature_data = legislature_data.annotate(expensed=Sum('expensed')).order_by('-expensed')

                    if legislature_data:
                        legislature_data = legislature_data[0]
                    else:
                        legislature_data = dict(expensed='0.')

                    p = PerNature(institution=institution,
                                  legislature=legislature,
                                  date_start=legislature_ranges['cdf'],
                                  date_end=legislature_ranges['cdt'],
                                  nature=nature,
                                  expensed=legislature_data['expensed'])
                    per_natures_to_create.append(p)

                # By Year
                for year in years:
                    print u'[%s] Consolidating nature %s totals for year %d…' % (institution.siglum, nature.name, year)
                    year_data = Expense.objects.filter(nature=nature)
                    year_data = year_data.filter(date__year=year)
                    year_data = filter_for_institution(year_data, institution)

                    # By Month
                    last_date = year_data and year_data.order_by('-date')[0].date or date.today()
                    for month in range(1, 13):
                        print u'[%s] Consolidating nature %s totals for %d-%d…' % (institution.siglum, nature.name, year, month)
                        month_date = date(year, month, 1)

                        if month_date >= last_date:
                            break

                        mdata = year_data.filter(date__month=month)
                        mdata = mdata.values('nature__id')
                        mdata = mdata.annotate(expensed=Sum('expensed')).order_by('-expensed')

                        if mdata:
                            mdata = mdata[0]
                        else:
                            mdata = dict(expensed='0.')

                        p = PerNatureByMonth(institution=institution,
                                             date=month_date,
                                             nature=nature,
                                             expensed=float(mdata['expensed']))
                        per_natures_by_month_to_create.append(p)

                    year_data = year_data.values('nature__id')
                    year_data = year_data.annotate(expensed=Sum("expensed"))

                    if year_data:
                        year_data = year_data[0]
                    else:
                        year_data = dict(expensed='0.')

                    p = PerNatureByYear(institution=institution,
                                        year=year,
                                        nature=nature,
                                        expensed=float(year_data['expensed']))
                    per_natures_by_year_to_create.append(p)

            PerNature.objects.bulk_create(per_natures_to_create)
            PerNatureByMonth.objects.bulk_create(per_natures_by_month_to_create)
            PerNatureByYear.objects.bulk_create(per_natures_by_year_to_create)

            # Legislator
            PerLegislator.objects.filter(institution=institution).delete()

            data = Expense.objects.all()
            data = filter_for_institution(data, institution)

            date_ranges = get_date_ranges_from_data(institution, data)

            data = data.values('mandate__legislator__id')
            data = data.annotate(expensed=Sum('expensed'))

            per_legislators_to_create = list()
            for item in data:
                legislator = Legislator.objects.get(id=int(item['mandate__legislator__id']))

                # Totals for Legislature
                for legislature in institution.legislature_set.all():
                    pargs = (institution.siglum, legislator.name, legislature.date_start.year, legislature.date_end.year)
                    print u'[%s] Consolidating legislator %s totals for legislature %d-%d…' % pargs

                    legislature_data = Expense.objects.filter(mandate__legislature=legislature)
                    legislature_data = legislature_data.filter(mandate__legislator=legislator)

                    legislature_ranges = get_date_ranges_from_data(institution, legislature_data)

                    legislature_data = legislature_data.values('mandate__legislator__id')
                    legislature_data = legislature_data.annotate(expensed=Sum('expensed')).order_by('-expensed')

                    if legislature_data:
                        legislature_data = legislature_data[0]
                    else:
                        legislature_data = dict(expensed='0.')

                    p = PerLegislator(institution=institution,
                                      legislature=legislature,
                                      date_start=date_ranges['cdf'],
                                      date_end=date_ranges['cdt'],
                                      legislator=legislator,
                                      expensed=legislature_data['expensed'])
                    per_legislators_to_create.append(p)

                print u'[%s] Consolidating totals for legislator %s…' % (institution.siglum, legislator.name)
                p = PerLegislator(institution=institution,
                                  date_start=date_ranges['cdf'],
                                  date_end=date_ranges['cdt'],
                                  legislator=legislator,
                                  expensed=item['expensed'])
                per_legislators_to_create.append(p)
            PerLegislator.objects.bulk_create(per_legislators_to_create)

        if 'agnostic' in args:
            # Institution-agnostic consolidations - biggest suppliers
            print u'Consolidating institution-agnostic totals…'

            BiggestSupplierForYear.objects.all().delete()
            years = [d.year for d in Expense.objects.dates('date', 'year')]
            for year in years:
                print u'Consolidating supplier totals for year %d…' % year
                data = Expense.objects.filter(date__year=year)
                data = data.values('supplier__id')
                data = data.annotate(expensed=Sum('expensed')).order_by('-expensed')

                biggest_suppliers_for_year_to_add = list()
                for item in data:
                    supplier = Supplier.objects.get(id=item['supplier__id'])

                    b = BiggestSupplierForYear(supplier=supplier,
                                               year=year,
                                               expensed=item['expensed'])
                    biggest_suppliers_for_year_to_add.append(b)
                BiggestSupplierForYear.objects.bulk_create(biggest_suppliers_for_year_to_add)



# -*- coding: utf-8 -*-
#
# Copyright (©) 2013 Marcelo Jorge Vieira <metal@alucinados.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.views.generic import TemplateView
from django.conf.urls import patterns, url
from django.views.defaults import page_not_found


urlpatterns = patterns(
    'montanha.views',

    # JSON queries
    url(r'^([^/]+)?/?q/all/?$', 'query_all', name='query-all'),
    url(r'^([^/]+)?/?q/supplier_all/?$', 'query_supplier_all', name='query-supplier-all'),
    url(r'^([^/]+)?/?q/legislator_all/?$', 'query_legislator_all', name='query-legislator-all'),

    url(r'^([^/]+)?/?all/?$', 'show_all', name='show-all'),

    url(r'^([^/]+)?/?per-nature/?$', 'show_per_nature', name='per-nature'),
    url(r'^([^/]+)?/?per-legislator/?$', 'show_per_legislator', name='per-legislator'),
    url(r'^([^/]+)?/?per-party/?$', 'show_per_party', name='per-party'),
    url(r'^([^/]+)?/?per-supplier/?$', 'show_per_supplier', name='per-supplier'),

    url(r'^detail-supplier/(\d+)/?$', 'show_supplier_overview', name='show-supplier-overview'),

    url(r'^([^/]+)?/?detail-legislator/(\d+)/?$', 'show_legislator_detail', name='show-legislator-detail'),
    url(r'^([^/]+)?/?detail-supplier/(\d+)/?$', 'show_supplier_detail', name='show-supplier-detail'),

    # favicon
    url(r'^favicon.ico$', page_not_found, name='page-not-found'),

    # Robots.txt
    (r'^robots\.txt$', TemplateView.as_view(template_name='robots.txt',
                                            content_type='text/plain')),

    # What is expenses?
    url(r'^o-que-e-verba-indenizatoria/?$', 'what_is_expenses', name='what-is-expenses'),

    # Contact us
    url(r'^fale-conosco/?$', 'contact_us', name='contact-us'),

    # Index
    url(r'^([^/]+)?/?$', 'show_index', name='index'),
)



# -*- coding: utf-8 -*-
#
# Copyright (©) 2013 Marcelo Jorge Vieira <metal@alucinados.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from parsley.decorators import parsleyfy
from captcha.fields import ReCaptchaField


@parsleyfy
class ContactUsForm(forms.Form):

    name = forms.CharField(
        label="Nome",
        required=True,
        widget=forms.TextInput(
            attrs={"placeholder": "Seu nome",
                   "data-trigger": "change",
                   "class": "span5"}))

    email = forms.EmailField(
        label="Email",
        required=True,
        widget=forms.TextInput(
            attrs={"placeholder": "Seu email",
                   "data-trigger": "change",
                   "class": "span5"}))

    message = forms.CharField(
        label="Mensagem",
        required=True,
        widget=forms.Textarea(
            attrs={"placeholder": "Sua mensagem",
                   "data-trigger": "change",
                   "class": "span5",
                   'rows': 5}))

    captcha = ReCaptchaField(attrs={'theme' : 'clean'})




# -*- coding: utf-8 -*-
#
# Copyright (©) 2013, Marcelo Jorge Vieira <metal@alucinados.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext as _
from django.contrib.sites.models import Site


class FAQ(models.Model):
    site = models.ForeignKey(
        Site,
        verbose_name=_('Site'),
        default=settings.SITE_ID)
    question = models.TextField(verbose_name=_('Question'), max_length=1000)
    answer = models.TextField(verbose_name=_('Answer'), max_length=5000)
    last_change = models.DateTimeField(
        verbose_name=_('Last Change'),
        auto_now=True)
    date_created = models.DateTimeField(
        verbose_name=_('Date Created'),
        auto_now_add=True)

    class Meta:
        verbose_name = _('FAQ')
        verbose_name_plural = _('FAQ')

    def __unicode__(self):
        return u"%s" % (self.question)



# -*- coding: utf-8 -*-
#
# Copyright (©) 2015, Marcelo Jorge Vieira
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import time
from datetime import datetime

import requests
from django.core.management.base import BaseCommand
from BeautifulSoup import BeautifulSoup, BeautifulStoneSoup

from montanha.models import Supplier


class Command(BaseCommand):
    args = "<source> [debug]"
    help = "Update supplier data"

    def post_process_uri(self, contents):
        return BeautifulSoup(
            contents,
            convertEntities=BeautifulStoneSoup.ALL_ENTITIES
        )

    def debug(self, message):
        message = message.encode('utf-8')

        if self.debug_enabled:
            print message

        if not hasattr(self, 'logfile'):
            self.logfile = open(self.__class__.__name__.lower() + '.log', 'a')

        timestamp = datetime.fromtimestamp(time.time()).strftime('%F:%H:%M:%S')
        self.logfile.write('%s %s\n' % (timestamp, message))

    def get_item(self, class_name):
        data = self.html \
            .find("p", {"class": class_name}) \
            .find("span", {"class": "text"})
        return data

    def handle(self, *args, **options):
        self.debug_enabled = False
        self.only_empty = False

        if "debug" in args:
            self.debug_enabled = True

        if "only-empty" in args:
            self.only_empty = True

        suppliers = Supplier.objects.all()
        for supplier in suppliers:
            if self.only_empty and supplier.address:
                continue

            time.sleep(10)

            try:
                url = 'http://www.cnpjbrasil.com/e/cnpj/t/%s' % (
                    supplier.identifier
                )
                response = requests.get(url)
            except Exception as e:
                self.debug(u'Error on get %s: %s' % (supplier, str(e)))
                continue

            if response.status_code != 200:
                self.debug(u'Error on get %s: Status code is %d' % (
                    supplier, response.status_code)
                )
                continue

            contents = self.post_process_uri(response.text)
            if not contents:
                self.debug(
                    u'Error on get %s: There are no contents' % supplier
                )
                continue

            self.html = contents.find(id='details')
            if not self.html:
                self.debug(u'Error on get %s: There are no details' % supplier)
                continue

            trade_name = self.get_item('fantasia')
            if trade_name:
                supplier.trade_name = trade_name.text

            date_opened = self.get_item('opened')
            format_date = datetime.strptime(date_opened.text, '%d/%m/%Y')
            if format_date:
                supplier.date_opened = format_date

            # FIXME
            address = self.get_item('address')
            for e in address.findAll('br'):
                e.replaceWith(' ')
            if address:
                supplier.address = re.sub('<[^<]+?>', '', str(address))

            juridical_nature = self.get_item('nj')
            if juridical_nature:
                supplier.juridical_nature = juridical_nature.text

            economic_activity = self.get_item('ae1')
            if economic_activity:
                supplier.main_economic_activity = economic_activity.text

            status = self.get_item('status')
            if status:
                supplier.status = True if status.text == 'Ativa' else False

            supplier.save()

            self.debug(u'Supplier %s updated!' % supplier)



# -*- coding: utf-8 -*-
#
# Copyright (©) 2010-2013 Estêvão Samuel Procópio
# Copyright (©) 2010-2013 Gustavo Noronha Silva
# Copyright (©) 2013 Marcelo Jorge Vieira
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import re
import datetime
from basecollector import BaseCollector
from datetime import datetime
from montanha.models import *


class ALMG(BaseCollector):
    def __init__(self, collection_runs, debug_enabled=False):
        super(ALMG, self).__init__(collection_runs, debug_enabled)

        try:
            institution = Institution.objects.get(siglum='ALMG')
        except Institution.DoesNotExist:
            institution = Institution(siglum='ALMG', name=u'Assembléia Legislativa do Estado de Minas Gerais')
            institution.save()

        try:
            self.legislature = Legislature.objects.all().filter(institution=institution).order_by('-date_start')[0]
        except IndexError:
            self.legislature = Legislature(institution=institution,
                                           date_start=datetime(2015, 1, 1),
                                           date_end=datetime(2018, 12, 31))
            self.legislature.save()

    def post_process_uri(self, contents):
        # The JSON returned by ALMG's web service uses the brazilian
        # locale for floating point numbers (uses , instead of .).
        data = re.sub(r"([0-9]+),([0-9]+)", r"\1.\2", contents)
        return json.loads(data)

    def try_name_disambiguation(self, name):
        if name == 'Luiz Henrique':
            try:
                return Legislator.objects.get(id=52), False
            except Legislator.DoesNotExist:
                return None, False

        return None, False

    def update_legislators(self):
        for situation in ['em_exercicio', 'que_exerceram_mandato']:
            legislators = self.retrieve_uri("http://dadosabertos.almg.gov.br/ws/deputados/%s?formato=json" % situation)["list"]
            for entry in legislators:
                try:
                    party = PoliticalParty.objects.get(siglum=entry["partido"])
                except PoliticalParty.DoesNotExist:
                    party = PoliticalParty(siglum=entry["partido"])
                    party.save()

                    self.debug("New party: %s" % unicode(party))

                legislator, created = self.try_name_disambiguation(entry['nome'])
                if not legislator:
                    legislator, created = Legislator.objects.get_or_create(name=entry['nome'])

                if created:
                    self.debug("New legislator: %s" % unicode(legislator))
                else:
                    self.debug("Found existing legislator: %s" % unicode(legislator))

                mandate = self.mandate_for_legislator(legislator, party, original_id=entry["id"])

    def update_legislators_data(self):

        mandates = self.legislature.mandate_set.all()
        for mandate in mandates:
            original_id = mandate.original_id
            uri = "http://dadosabertos.almg.gov.br/ws/deputados/%s?formato=json" % original_id
            entry = self.retrieve_uri(uri)["deputado"]

            self.debug("Legislator %s" % unicode(mandate.legislator))

            if "sexo" in entry:
                mandate.legislator.gender = entry["sexo"]

            if "sitePessoal" in entry:
                mandate.legislator.site = entry["sitePessoal"]

            if "vidaProfissionalPolitica" in entry:
                mandate.legislator.about = entry["vidaProfissionalPolitica"]

            if "emails" in entry and entry["emails"]:
                email = entry["emails"][0]['endereco']
                mandate.legislator.email = "%s%s" % (email, "@almg.gov.br")

            if "dataNascimento" in entry:
                date_of_birth = entry["dataNascimento"]
                # Removed crazy char º
                crazy_char = "º".decode("utf-8")
                if crazy_char in date_of_birth:
                    date_of_birth = date_of_birth.replace(crazy_char, "")
                date_of_birth = datetime.strptime(date_of_birth,
                                                  "%d/%m/%Y").date()
                mandate.legislator.date_of_birth = date_of_birth

            mandate.legislator.save()

    def update_data_for_year(self, mandate, year):
        self.debug("Updating data for year %d" % year)
        for month in range(1, 13):
            self.update_data_for_month(mandate, year, month)

    def update_data_for_month(self, mandate, year, month):
        self.debug("Updating data for %d-%d - %s" % (year, month, unicode(mandate)))
        uri = "http://dadosabertos.almg.gov.br/ws/prestacao_contas/verbas_indenizatorias/deputados/%s/%d/%d?formato=json" % (mandate.original_id, year, month)
        for entry in self.retrieve_uri(uri)["list"]:
            try:
                nature = ExpenseNature.objects.get(original_id=entry["codTipoDespesa"])
            except ExpenseNature.DoesNotExist:
                nature = ExpenseNature(original_id=entry["codTipoDespesa"], name=entry["descTipoDespesa"])
                nature.save()

            for details in entry["listaDetalheVerba"]:
                cnpj = self.normalize_cnpj_or_cpf(details["cpfCnpj"])
                try:
                    supplier = Supplier.objects.get(identifier=cnpj)
                except Supplier.DoesNotExist:
                    supplier = Supplier(identifier=cnpj, name=details["nomeEmitente"])
                    supplier.save()

                if "descDocumento" in details:
                    number = details["descDocumento"]
                else:
                    self.debug("No document number, using reference date.")
                    number = details["dataReferencia"]["$"]

                date = details["dataEmissao"]["$"]
                value = details["valorDespesa"]
                expensed = details["valorReembolsado"]

                expense = ArchivedExpense(original_id=details["id"],
                                          number=number,
                                          nature=nature,
                                          date=date,
                                          value=value,
                                          expensed=expensed,
                                          mandate=mandate,
                                          supplier=supplier,
                                          collection_run=self.collection_run)
                expense.save()

                self.debug("New expense found: %s" % unicode(expense))
